#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Core {
  public abstract class CallNodeBase : ItemBase, ICallNode {
    [Storable]
    private CallGraph parentGraph;

    [Storable]
    private Dictionary<string, string> myNameMappings;
    public IEnumerable<KeyValuePair<string, string>> NameMappings {
      get { return myNameMappings; }
    }
    public IList<ICallNode> SuccessorNodes {
      get { return parentGraph != null ? parentGraph.GetSuccessorNodes(this) : null; }
    }
    public bool InitialCallNode {
      get { return parentGraph != null ? parentGraph.InitialCallNode == this : false; }
      set {
        if (parentGraph != null)
          parentGraph.InitialCallNode = this;
      }
    }

    protected CallNodeBase()
      : base() {
      parentGraph = null;
      myNameMappings = new Dictionary<string, string>();
    }
    protected CallNodeBase(CallGraph parentGraph)
      : this() {
      this.parentGraph = parentGraph;
      parentGraph.AddCallNode(this);
    }

    public override ICloneable Clone(ICloner cloner) {
      CallNodeBase clone = (CallNodeBase)base.Clone(cloner);
      foreach (KeyValuePair<string, string> nameMapping in myNameMappings)
        clone.AddNameMapping(nameMapping.Key, nameMapping.Value);
      return clone;
    }

    public void AddNameMapping(string name, string alias) {
      RemoveNameMapping(name);
      myNameMappings.Add(name, alias);
      OnNameMappingAdded(name, alias);
    }
    public void RemoveNameMapping(string name) {
      if (myNameMappings.ContainsKey(name)) {
        string alias = myNameMappings[name];
        myNameMappings.Remove(name);
        OnNameMappingRemoved(name, alias);
      }
    }
    protected void ClearNameMappings() {
      string[] names = new string[myNameMappings.Count];
      myNameMappings.Keys.CopyTo(names, 0);
      foreach (string name in names)
        RemoveNameMapping(name);
    }
    public string MapName(string name) {
      string alias = name;
      myNameMappings.TryGetValue(name, out alias);
      if (parentGraph != null)
        alias = parentGraph.MapName(alias);
      return alias;
    }

    public event EventHandler<EventArgs<string, string>> NameMappingAdded;
    protected virtual void OnNameMappingAdded(string name, string alias) {
      if (NameMappingAdded != null)
        NameMappingAdded(this, new EventArgs<string, string>(name, alias));
    }
    public event EventHandler<EventArgs<string, string>> NameMappingRemoved;
    protected virtual void OnNameMappingRemoved(string name, string alias) {
      if (NameMappingRemoved != null)
        NameMappingRemoved(this, new EventArgs<string, string>(name, alias));
    }
  }
}
